{ ============================================================================= }
{                         UNIT UOceanWaterPlaneDemo                             }
{                                                                               }
{ Description: This demonstrates a modification to the GLWaterPlane unit.       }
{ The GLWaterPlane unit has been modified to allow for sin wave's that simulate }
{ the sort of waves you might see on a calm ocean or lake.                      }
{                                                                               }
{ There are two new properties on the water plane. These include                }
{ smoothwaveheight which defines the height of a smoothwave (relative units)    }
{ and smoothwavefrequency (relative also). These waves dont interact with the   }
{ rain waves but the two types of waves can be present at the same time.        }
{                                                                               }
{ Note also that it is quite fast (faster than the rain) and have tried to      }
{ make it fast so that large oceans an be simulated.                            }
{                                                                               }
{ Created June 2004 Skinhat                                                     }
{                                                                               }
{ ============================================================================= }
unit uOceanWaterPlaneDemo;
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, GLScene, GLObjects, GLMisc, GLWin32Viewer, GLWaterPlane,
  GLCadencer, ExtCtrls, Jpeg, GLTexture, GLUserShader, OpenGL1x,
  VectorGeometry, GLGraph, VectorTypes, GLState, StdCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    GLScene1: TGLScene;
    GLSceneViewer1: TGLSceneViewer;
    GLCamera1: TGLCamera;
    GLCadencer1: TGLCadencer;
    Timer1: TTimer;
    GLWaterPlane1: TGLWaterPlane;
    GLMaterialLibrary1: TGLMaterialLibrary;
    GLUserShader1: TGLUserShader;
    GLDirectOpenGL1: TGLDirectOpenGL;
    GLLightSource1: TGLLightSource;
    GLPlane1: TGLPlane;
    GLPlane2: TGLPlane;
    Panel1: TPanel;
    TrackBar1: TTrackBar;
    Label2: TLabel;
    TrackBar2: TTrackBar;
    Label1: TLabel;
    cbRain: TCheckBox;
    procedure Timer1Timer(Sender: TObject);
    procedure GLCadencer1Progress(Sender: TObject; const deltaTime,
      newTime: Double);
    procedure FormCreate(Sender: TObject);
    procedure GLUserShader1DoApply(Sender: TObject;
      var rci: TRenderContextInfo);
    procedure GLSceneViewer1BeforeRender(Sender: TObject);
    procedure GLDirectOpenGL1Render(Sender : TObject; var rci: TRenderContextInfo);
    procedure GLUserShader1DoUnApply(Sender: TObject; Pass: Integer;
      var rci: TRenderContextInfo; var Continue: Boolean);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbRainClick(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure TrackBar2Change(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    reflectionToggle : Boolean;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}


procedure TForm1.FormCreate(Sender: TObject);
const
  scale=3;
begin

   glwaterplane1.Scale.x:=glwaterplane1.Scale.x*scale;
   glwaterplane1.Scale.y:=glwaterplane1.Scale.y*scale;
   glwaterplane1.Scale.z:=glwaterplane1.Scale.z*scale;
   //glwaterplane1.RainTimeInterval:=100;
   glwaterplane1.SmoothwaveFrequency:=1;
   //glwaterplane1.Smoothwaveheight:=5;
   // Load the cube map which is used both for environment and as reflection texture
   with GLMaterialLibrary1.Materials[0].Material.Texture do begin
      ImageClassName:=TGLCubeMapImage.ClassName;
      with Image as TGLCubeMapImage do begin
         // Load all 6 texture map components of the cube map
         // The 'PX', 'NX', etc. refer to 'positive X', 'negative X', etc.
         // and follow the RenderMan specs/conventions

         Picture[cmtPX].LoadFromFile('clouds.jpg');
         Picture[cmtNX].LoadFromFile('clouds.jpg');
         Picture[cmtPY].LoadFromFile('clouds.jpg');
         Picture[cmtNY].LoadFromFile('clouds.jpg');
         Picture[cmtPZ].LoadFromFile('clouds.jpg');
         Picture[cmtNZ].LoadFromFile('clouds.jpg');
      end;
   end;
  glmateriallibrary1.Materials[1].Material.texture.image.loadfromfile('clouds.jpg');
  //glmateriallibrary1.Materials[2].Material.texture.image.loadfromfile('seafloor.jpg');

  glsceneviewer1.buffer.Backgroundcolor:=$845D45;
  glmateriallibrary1.Materials[1].Material.frontproperties.emission.aswincolor:=$997851;

  glplane2.Material.Texture.Image.LoadFromFile('seafloor.jpg');


end;

procedure TForm1.GLUserShader1DoApply(Sender: TObject;
  var rci: TRenderContextInfo);
var
   cubeMapMode : Integer;
begin
   // Here is the shader trick: the same cubemap is used in reflection mode
   // for the pond, and in normal mode for the environment sphere
   // Our basic user shader takes care of that.
   if reflectionToggle then begin
      cubeMapMode:=GL_REFLECTION_MAP_ARB;
      rci.GLStates.SetGLState(stBlend);
   end else begin
      cubeMapMode:=GL_NORMAL_MAP_ARB;
      rci.GLStates.UnSetGLState(stBlend);
   end;

   glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, cubeMapMode);
   glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, cubeMapMode);
   glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, cubeMapMode);

   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
end;

procedure TForm1.GLUserShader1DoUnApply(Sender: TObject; Pass: Integer;
  var rci: TRenderContextInfo; var Continue: Boolean);
begin
   rci.GLStates.UnSetGLState(stBlend);
end;

procedure TForm1.GLSceneViewer1BeforeRender(Sender: TObject);
begin
   reflectionToggle:=False;   // toggle for environment sphere
end;

procedure TForm1.GLDirectOpenGL1Render(Sender : TObject; var rci: TRenderContextInfo);
begin
   reflectionToggle:=True;    // toggle for pond/water plane
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
   Caption:= GLSceneViewer1.FramesPerSecondText
            +Format(' / %.3f ms', [GLWaterPlane1.LastIterationStepTime*1000]);
   GLSceneViewer1.ResetPerformanceMonitor;
end;

procedure TForm1.GLCadencer1Progress(Sender: TObject; const deltaTime,
  newTime: Double);
begin
   GLSceneViewer1.Invalidate;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  factor:double;
begin

  if (ssshift in shift) then
    factor:=4
  else
    factor:=2;
  case key of
    35,40: glcamera1.move(-0.3*factor);
    37: begin
        glcamera1.turn(-5*factor);

      end;
    39: begin
        glcamera1.turn(5*factor);

      end;
    38,65,16: glcamera1.move(0.3*factor);
    33:  glcamera1.pitch(1*factor);
    34:  glcamera1.pitch(-1*factor);

  end;
end;

procedure TForm1.cbRainClick(Sender: TObject);
begin
  activecontrol:=nil;
  if cbrain.checked then
    glwaterplane1.RainTimeInterval:=1
  else
    glwaterplane1.RainTimeInterval:=0;
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  activecontrol:=nil;
  glWaterplane1.SmoothwaveHeight:=  trackbar1.Position;
end;

procedure TForm1.TrackBar2Change(Sender: TObject);
begin
  activecontrol:=nil;
  glWaterplane1.SmoothwaveFrequency:=  trackbar2.Position;

end;

end.
